@@ -6,7 +6,7 @@ Python implementation of jsonurl, an alternative format for JSON data model
 See https://jsonurl.org/ and https://github.com/jsonurl/specification/
 """
 
-__version__ = "0.3.2"
+__version__ = "0.4.0"
 
 import re
 import sys
@@ -43,6 +43,13 @@ class CommonOpts:
     See `spec section 2.9.2 <https://github.com/jsonurl/specification/#292-implied-objects>`_
     """
 
+    distinguish_empty_list_dict: bool = False
+    """Distinguish between empty list and empty dict
+
+    Use ``(:)`` for empty dict and ``()`` for empty list as per `spec section 2.9.5
+    <https://github.com/jsonurl/specification/#295-empty-objects-and-arrays>`_
+    """
+
     aqf: bool = False
     """Address bar Query string Friendly
 
@@ -54,7 +61,21 @@ class CommonOpts:
 @_dataclass_kwonly
 class DumpOpts(CommonOpts):
     """
-    Options for `dumps`
+    Options for `jsonurl_py.dumps`
+    """
+
+    safe: str = ""
+    """
+    Additional characters considered safe and not requiring percent-encoding.
+
+    This is similar to the safe argument to `urllib.parse.quote_plus`.
+
+    By default only the characters ``[a-zA-Z0-9_.-~]`` are safe. Additional
+    characters that can be marked safe are: ``!$*/;?@``. In AQF mode the
+    characters "'" can also be marked as safe.
+
+    In AQF mode the characters `!(),:` are considered safe by default so that
+    output is readable but on input they are also recognized in encoded form.
     """
 
 
@@ -80,7 +101,7 @@ def _dump_str(arg: str, opts: DumpOpts) -> str:
             return "!e"
         if RE_NUMBER.match(arg):
             return "!" + arg
-        return quote_plus(arg, safe="(),:!").translate(
+        return quote_plus(arg, safe=opts.safe + "(),:!").translate(
             {
                 ord("!"): "!!",
                 ord("("): "!(",
@@ -100,7 +121,7 @@ def _dump_str(arg: str, opts: DumpOpts) -> str:
             return "''"
         if RE_NUMBER.match(arg):
             return "'" + arg + "'"
-        return quote_plus(arg)
+        return quote_plus(arg, safe=opts.safe)
 
 
 def _dump_any(arg: Any, opts: DumpOpts) -> str:
@@ -119,7 +140,10 @@ def _dump_any(arg: Any, opts: DumpOpts) -> str:
     if isinstance(arg, list):
         return "(" + _dump_list_data(arg, opts) + ")"
     if isinstance(arg, dict):
-        return "(" + _dump_dict_data(arg, opts) + ")"
+        if len(arg) == 0 and opts.distinguish_empty_list_dict:
+            return "(:)"
+        else:
+            return "(" + _dump_dict_data(arg, opts) + ")"
     raise TypeError(f"Bad value {arg!r} of type {type(arg)}")
 
 
@@ -135,6 +159,8 @@ def dumps(
     implied_list: bool = False,
     implied_dict: bool = False,
     aqf: bool = False,
+    safe: str = "",
+    distinguish_empty_list_dict: bool = False,
 ) -> str:
     ...
 
@@ -149,6 +175,7 @@ def dumps(arg: Any, opts=None, **kw) -> str:
         opts = DumpOpts(**kw)
     elif kw:
         raise ValueError("Either opts or kw, not both")
+    check_can_mark_safe(opts.safe, opts.aqf)
 
     if opts.implied_dict:
         return _dump_dict_data(arg, opts)
@@ -157,6 +184,16 @@ def dumps(arg: Any, opts=None, **kw) -> str:
     return _dump_any(arg, opts)
 
 
+def check_can_mark_safe(safe: str, aqf=False):
+    """Check if a string can be marked as safe for jsonurl"""
+    for c in safe:
+        if c in "!$*/;?@":
+            continue
+        if aqf and c == "'":
+            continue
+        raise ValueError(f"Can't mark character {c!r} as safe")
+
+
 @_dataclass_kwonly
 class LoadOpts(CommonOpts):
     """
@@ -434,8 +471,20 @@ def _load_any(arg: str, pos: int, opts: LoadOpts) -> Tuple[Any, int]:
         if char == "(":
             first_val, pos = _load_any(arg, pos, opts)
             return _load_list(arg, pos, first_val, opts)
+        if opts.distinguish_empty_list_dict and char == ":":
+            pos += 1
+            if pos == len(arg):
+                raise ParseError("Unterminated empty composite, expected )")
+            char = arg[pos]
+            if char == ")":
+                return {}, pos + 1
+            else:
+                raise ParseError("Unterminated empty composite, expected )")
         if char == ")":
-            return {}, pos + 1
+            if opts.distinguish_empty_list_dict:
+                return [], pos + 1
+            else:
+                return {}, pos + 1
         return _load_comp(arg, pos, opts)
     else:
         return _load_atom(arg, pos, opts)
@@ -485,6 +534,7 @@ def loads(
     implied_dict: bool = False,
     implied_list: bool = False,
     aqf: bool = False,
+    distinguish_empty_list_dict: bool = False,
 ) -> Any:
     ...
 
