@@ -5,7 +5,7 @@ set -e -x
 exec docker run \
     --interactive \
     --tty \
-    --volume $(pwd):/workdir \
+    --volume "$(pwd):/workdir" \
     --workdir /workdir \
     fkrull/multi-python \
     "$@"
