@@ -13,12 +13,17 @@ import sys
 sys.path.insert(0, os.path.abspath(".."))
 
 extensions = [
-    "sphinx.ext.autodoc",
     "sphinxarg.ext",
+    "sphinx.ext.autodoc",
+    "sphinx.ext.intersphinx",
 ]
 
+intersphinx_mapping = {
+    "python": ("https://docs.python.org/3", None),
+}
+
 # Preserve order in source code
 autodoc_member_order = "bysource"
 
 html_theme = "sphinx_rtd_theme"
-default_role = "py:obj"
+default_role = "any"
